import os
from docx import Document
from docx.shared import Inches
from tqdm import tqdm
from PIL import Image


def get_images_from_folder(folder):
    """Returns a list of image file paths from the given folder."""
    valid_extensions = (".jpg", ".jpeg", ".png", ".bmp", ".gif")
    return [os.path.join(folder, f) for f in os.listdir(folder) if f.lower().endswith(valid_extensions)]


def add_image_keeping_ratio(run, img_path, max_width=1.5, max_height=1.5):
    """Adds an image to a Word document while maintaining its original aspect ratio."""
    with Image.open(img_path) as img:
        width, height = img.size
        ratio = min(max_width / width, max_height / height)
        new_width = width * ratio
        new_height = height * ratio
        run.add_picture(img_path, width=Inches(new_width), height=Inches(new_height))


def create_docx_from_images(image_paths, output_doc="output.docx"):
    """Generates a DOCX file with 20 images per page, keeping aspect ratio."""
    if not image_paths:
        print("No images found in the current directory.")
        return

    doc = Document()
    images_per_page = 20
    total_images = len(image_paths)

    for i in tqdm(range(0, total_images, images_per_page), desc="Processing Images"):
        page_images = image_paths[i : i + images_per_page]
        table = doc.add_table(rows=4, cols=5)

        row_idx, col_idx = 0, 0
        for img_path in page_images:
            if os.path.exists(img_path):
                cell = table.cell(row_idx, col_idx)
                cell_paragraph = cell.paragraphs[0]
                run = cell_paragraph.add_run()

                add_image_keeping_ratio(run, img_path)
                cell.paragraphs[0].alignment = 1
                for p in cell.paragraphs:
                    p.alignment = 1

            col_idx += 1
            if col_idx == 5:
                col_idx = 0
                row_idx += 1

        doc.add_paragraph().add_run().add_break()

    doc.save(output_doc)
    print(f"DOCX file created successfully: {output_doc}")


def main():
    """Main function to find images and create a DOCX file."""
    current_folder = os.getcwd()
    images = get_images_from_folder(current_folder)

    if images:
        create_docx_from_images(images)
    else:
        print("No image files found in the current folder.")


if __name__ == "__main__":
    main()
